package ink.wgink.service.controller.api;

import ink.wgink.common.base.DefaultBaseController;
import ink.wgink.exceptions.ParamsException;
import ink.wgink.interfaces.consts.ISystemConstant;
import ink.wgink.pojo.ListPage;
import ink.wgink.pojo.dtos.ZTreeDTO;
import ink.wgink.pojo.dtos.menu.MenuDTO;
import ink.wgink.pojo.result.ErrorResult;
import ink.wgink.pojo.result.SuccessResult;
import ink.wgink.pojo.result.SuccessResultList;
import ink.wgink.service.pojo.vos.MenuVO;
import ink.wgink.service.service.IMenuService;
import io.swagger.annotations.*;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @ClassName: MenuController
 * @Description: 菜单
 * @Author: wenc
 * @Date: 2018/12/27 10:09 PM
 * @Version: 1.0
 * @MenuUrl: admin/menu/gomenus
 **/
@Api(tags = ISystemConstant.API_TAGS_SYSTEM_PREFIX + "菜单信息管理接口")
@RestController
@RequestMapping(ISystemConstant.API_PREFIX + "/menu")
public class MenuController extends DefaultBaseController {

    public static final String SLASH = "/";
    @Autowired
    private IMenuService menuService;

    @ApiOperation(value = "保存菜单", notes = "保存菜单接口")
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @PostMapping("savemenu")
    public SuccessResult saveMenu(@RequestBody MenuVO menuVO) {
        Map<String, Object> params = new HashMap<>(16);
        checkParams(menuVO, params);
        return menuService.saveMenu(params);
    }

    @ApiOperation(value = "菜单删除", notes = "通过id列表批量删除菜单接口")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "菜单ID列表，用下划线分隔", paramType = "path")
    })
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @DeleteMapping("removemenu/{ids}")
    public SuccessResult removeMenu(@PathVariable("ids") String ids) {
        Map<String, Object> params = new HashMap<>(0);
        params.put("menuIds", ids);
        return menuService.removeMenu(params);
    }

    @ApiOperation(value = "修改菜单", notes = "修改菜单接口")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "menuId", value = "菜单ID", paramType = "path")
    })
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @PutMapping("updatemenu/{menuId}")
    public SuccessResult updateMenu(@PathVariable("menuId") String menuId, @RequestBody MenuVO menuVO) {
        Map<String, Object> params = new HashMap<>(16);
        params.put("menuId", menuId);
        checkParams(menuVO, params);
        return menuService.updateMenu(params);
    }

    /**
     * 参数校验
     *
     * @param menuVO
     * @param params
     * @throws ParamsException
     */
    private void checkParams(MenuVO menuVO, Map<String, Object> params) {
        if (StringUtils.isBlank(menuVO.getMenuParentId())) {
            throw new ParamsException("父ID不能为空");
        }
        params.put("menuParentId", menuVO.getMenuParentId());

        if (StringUtils.isBlank(menuVO.getMenuName())) {
            throw new ParamsException("菜单名称不能为空");
        }
        params.put("menuName", menuVO.getMenuName());

        if (StringUtils.isBlank(menuVO.getMenuOrder())) {
            throw new ParamsException("菜单排序不能为空");
        }
        params.put("menuOrder", menuVO.getMenuOrder());

        if (menuVO.getMenuType() == null) {
            throw new ParamsException("菜单类别不能为空或格式错误，格式为数字");
        }
        params.put("menuType", menuVO.getMenuType());

        if (menuVO.getMenuStatus() == null) {
            throw new ParamsException("菜单状态不能为空或格式错误，格式为数字");
        }
        params.put("menuStatus", menuVO.getMenuStatus());

        if (StringUtils.isBlank(menuVO.getMenuUrl())) {
            params.put("menuUrl", "javascript:void(0);");
        } else if (StringUtils.equals(menuVO.getMenuUrl(), "javascript:void(0);")) {
            params.put("menuUrl", menuVO.getMenuUrl());
        } else {
            params.put("menuUrl", addSlash(menuVO.getMenuUrl()));
        }

        if (menuVO.getOpenType() == null) {
            throw new ParamsException("打开方式不能为空或格式错误，格式为数字");
        }
        params.put("openType", menuVO.getOpenType());

        params.put("apiPrefix", addSlash(menuVO.getApiPrefix()));
        params.put("resourcePrefix", addSlash(menuVO.getResourcePrefix()));
        params.put("routePrefix", addSlash(menuVO.getRoutePrefix()));

        if (StringUtils.isBlank(menuVO.getMenuIcon())) {
            params.put("menuIcon", "fa-icon-color-white fa fa-list");
        } else {
            params.put("menuIcon", menuVO.getMenuIcon());
        }
    }

    /**
     * 添加斜杠
     *
     * @param prefix
     * @return
     */
    private String addSlash(String prefix) {
        if (!StringUtils.isEmpty(prefix)) {
            if (prefix.toLowerCase().startsWith("http://") || prefix.toLowerCase().startsWith("https://")) {
                return prefix;
            }
            if (!StringUtils.startsWith(prefix, SLASH)) {
                prefix = SLASH + prefix;
            }
            if (StringUtils.endsWith(prefix, SLASH)) {
                prefix = prefix.substring(0, prefix.length() - 1);
            }
        }
        return prefix;
    }

    @ApiOperation(value = "菜单详情", notes = "菜单详情接口")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "menuId", value = "菜单ID", paramType = "path")
    })
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @GetMapping("getmenu/{menuId}")
    public MenuDTO getMenu(@PathVariable("menuId") String menuId) {
        Map<String, Object> params = getParams();
        params.put("menuId", menuId);
        return menuService.getMenu(params);
    }

    @ApiOperation(value = "菜单列表", notes = "菜单列表接口")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "menuParentId", value = "菜单父ID", paramType = "path")
    })
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @GetMapping("listmenus/{menuParentId}")
    public List<MenuDTO> listMenus(@PathVariable("menuParentId") String menuParentId) {
        Map<String, Object> params = requestParams();
        params.put("menuParentId", menuParentId);
        return menuService.listMenuAllJson(params);
    }

    @ApiOperation(value = "zTree列表", notes = "zTree列表接口")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "父ID", paramType = "query", dataType = "String")
    })
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @GetMapping("listztreemenus")
    public List<ZTreeDTO> listZTreeMenus() {
        Map<String, Object> params = requestParams();
        String menuParentId = "0";
        if (!StringUtils.isBlank(params.get("id") == null ? null : params.get("id").toString())) {
            menuParentId = params.get("id").toString();
        }
        params.put("menuParentId", menuParentId);
        return menuService.listZTreeMenus(params);
    }

    @ApiOperation(value = "easyui菜单列表", notes = "easyui菜单列表接口")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "parentId", value = "上级ID", paramType = "query", dataType = "String"),
            @ApiImplicitParam(name = "page", value = "当前页码", paramType = "query", dataType = "int", defaultValue = "1"),
            @ApiImplicitParam(name = "rows", value = "显示数量", paramType = "query", dataType = "int", defaultValue = "20"),
            @ApiImplicitParam(name = "keywords", value = "关键字", paramType = "query", dataType = "String"),
            @ApiImplicitParam(name = "startTime", value = "开始时间", paramType = "query", dataType = "String"),
            @ApiImplicitParam(name = "endTime", value = "结束时间", paramType = "query", dataType = "String")
    })
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @GetMapping("listpagemenus")
    public SuccessResultList<List<MenuDTO>> listPageMenus(ListPage page) {
        Map<String, Object> params = requestParams();
        String menuParentId = "0";
        if (!StringUtils.isBlank(params.get("parentId") == null ? null : params.get("parentId").toString())) {
            menuParentId = params.get("parentId").toString();
        }
        params.put("menuParentId", menuParentId);
        page.setParams(params);
        return menuService.listPageMenus(page);
    }

}
